@@ -8,28 +8,51 @@ import functools
 import itertools as itt
 from abc import ABC, abstractmethod
 from dataclasses import dataclass, field
-from typing import Callable, Iterable, Sequence, Set, Tuple, TypeVar, Union
+from operator import attrgetter
+from typing import Callable, Iterable, Optional, Sequence, Set, Tuple, TypeVar, Union
 
 __all__ = [
-    'Variable',
-    'Intervention',
-    'CounterfactualVariable',
-    'Distribution',
-    'P',
-    'Probability',
-    'Sum',
-    'Product',
-    'Fraction',
-    'Expression',
-    'One',
-    'Q',
-    'QFactor',
-    'A', 'B', 'C', 'D', 'R', 'S', 'T', 'W', 'X', 'Y', 'Z',
-    'V1', 'V2', 'V3', 'V4', 'V5', 'V6',
-    'Z1', 'Z2', 'Z3', 'Z4', 'Z5', 'Z6',
+    "Variable",
+    "Intervention",
+    "CounterfactualVariable",
+    "Distribution",
+    "P",
+    "Probability",
+    "Sum",
+    "Product",
+    "Fraction",
+    "Expression",
+    "One",
+    "Q",
+    "QFactor",
+    "A",
+    "B",
+    "C",
+    "D",
+    "R",
+    "S",
+    "T",
+    "W",
+    "X",
+    "Y",
+    "Z",
+    "V1",
+    "V2",
+    "V3",
+    "V4",
+    "V5",
+    "V6",
+    "Z1",
+    "Z2",
+    "Z3",
+    "Z4",
+    "Z5",
+    "Z6",
+    # Helpers
+    "ensure_ordering",
 ]
 
-X = TypeVar('X')
+X = TypeVar("X")
 XSeq = Union[X, Sequence[X]]
 
 
@@ -39,7 +62,9 @@ def _upgrade_variables(variables: XSeq[Variable]) -> Tuple[Variable, ...]:
 
 def _to_interventions(variables: Sequence[Variable]) -> Tuple[Intervention, ...]:
     return tuple(
-        variable if isinstance(variable, Intervention) else Intervention(name=variable.name, star=False)
+        variable
+        if isinstance(variable, Intervention)
+        else Intervention(name=variable.name, star=False)
         for variable in variables
     )
 
@@ -54,7 +79,7 @@ class _Mathable(ABC):
         """Output this DSL object in the LaTeX string format."""
 
     def _repr_latex_(self) -> str:  # hack for auto-display of latex in jupyter notebook
-        return f'${self.to_latex()}$'
+        return f"${self.to_latex()}$"
 
     def __str__(self) -> str:
         return self.to_text()
@@ -76,8 +101,18 @@ class Variable(_Mathable):
     name: str
 
     def __post_init__(self):
-        if self.name in {'P', 'Q'}:
-            raise ValueError(f'trust me, {self.name} is a bad variable name.')
+        if self.name in {"P", "Q"}:
+            raise ValueError(f"trust me, {self.name} is a bad variable name.")
+
+    @classmethod
+    def norm(cls, name: Union[str, Variable]) -> Variable:
+        """Automatically upgrade a string to a variable."""
+        if isinstance(name, str):
+            return Variable(name)
+        elif isinstance(name, Variable):
+            return name
+        else:
+            raise TypeError
 
     def to_text(self) -> str:
         """Output this variable in the internal string format."""
@@ -121,7 +156,7 @@ class Variable(_Mathable):
                 parents=_upgrade_variables(parents),
             )
         elif parents.is_conditioned():
-            raise TypeError('can not be given a distribution that has conditionals')
+            raise TypeError("can not be given a distribution that has conditionals")
         else:
             # The parents variable is actually a Distribution instance with no parents,
             #  so its children become the parents for the new Markov Kernel distribution
@@ -181,11 +216,11 @@ class Intervention(Variable):
 
     def to_text(self) -> str:
         """Output this intervention variable in the internal string format."""
-        return f'{self.name}*' if self.star else self.name
+        return f"{self.name}*" if self.star else self.name
 
     def to_latex(self) -> str:
         """Output this intervention variable in the LaTeX string format."""
-        return f'{self.name}^*' if self.star else self.name
+        return f"{self.name}^*" if self.star else self.name
 
     def invert(self) -> Intervention:
         """Create an :class:`Intervention` variable that is different from what was observed (with a star)."""
@@ -208,23 +243,23 @@ class CounterfactualVariable(Variable):
 
     def __post_init__(self):
         if not self.interventions:
-            raise ValueError('should give at least one intervention')
+            raise ValueError("should give at least one intervention")
         for intervention in self.interventions:
             if not isinstance(intervention, Intervention):
                 raise TypeError(
-                    f'only Intervention instances are allowed.'
-                    f' Got: ({intervention.__class__.__name__}) {intervention}',
+                    f"only Intervention instances are allowed."
+                    f" Got: ({intervention.__class__.__name__}) {intervention}",
                 )
 
     def to_text(self) -> str:
         """Output this counterfactual variable in the internal string format."""
         intervention_latex = _list_to_text(self.interventions)
-        return f'{self.name}_{{{intervention_latex}}}'
+        return f"{self.name}_{{{intervention_latex}}}"
 
     def to_latex(self) -> str:
         """Output this counterfactual variable in the LaTeX string format."""
         intervention_latex = _list_to_latex(self.interventions)
-        return f'{self.name}_{{{intervention_latex}}}'
+        return f"{self.name}_{{{intervention_latex}}}"
 
     def intervene(self, variables: XSeq[Variable]) -> CounterfactualVariable:
         """Intervene on this counterfactual variable with the given variable(s).
@@ -244,7 +279,9 @@ class CounterfactualVariable(Variable):
             interventions=(*self.interventions, *_interventions),
         )
 
-    def _raise_for_overlapping_interventions(self, interventions: Iterable[Intervention]) -> None:
+    def _raise_for_overlapping_interventions(
+        self, interventions: Iterable[Intervention]
+    ) -> None:
         """Raise an error if any of the given variables are already listed in interventions in this counterfactual.
 
         :param interventions: Interventions to check for overlap
@@ -256,7 +293,9 @@ class CounterfactualVariable(Variable):
             if old.name == new.name
         }
         if overlaps:
-            raise ValueError(f'Overlapping interventions in new interventions: {overlaps}')
+            raise ValueError(
+                f"Overlapping interventions in new interventions: {overlaps}"
+            )
 
     def invert(self) -> Intervention:
         """Raise an error, since counterfactuals can't be inverted the same as normal variables or interventions."""
@@ -278,18 +317,18 @@ class Distribution(_Mathable):
 
     def __post_init__(self):
         if isinstance(self.children, (list, Variable)):
-            raise TypeError
+            raise TypeError(f"children of wrong type: {type(self.children)}")
         if isinstance(self.parents, (list, Variable)):
             raise TypeError
         if not self.children:
-            raise ValueError('distribution must have at least one child')
+            raise ValueError("distribution must have at least one child")
 
     def to_text(self) -> str:
         """Output this distribution in the internal string format."""
         children = _list_to_text(self.children)
         if self.parents:
             parents = _list_to_text(self.parents)
-            return f'{children}|{parents}'
+            return f"{children}|{parents}"
         else:
             return children
 
@@ -298,7 +337,7 @@ class Distribution(_Mathable):
         children = _list_to_latex(self.children)
         if self.parents:
             parents = _list_to_latex(self.parents)
-            return f'{children}|{parents}'
+            return f"{children}|{parents}"
         else:
             return children
 
@@ -310,6 +349,12 @@ class Distribution(_Mathable):
         """Return if this distribution a markov kernel -> one child variable and one or more conditionals."""
         return len(self.children) == 1
 
+    def uncondition(self) -> Distribution:
+        """Return a new distribution that is not conditioned on the parents."""
+        return Distribution(
+            children=(*self.children, *self.parents),
+        )
+
     def joint(self, children: XSeq[Variable]) -> Distribution:
         """Create a new distribution including the given child variables.
 
@@ -341,13 +386,16 @@ class Distribution(_Mathable):
                 parents=(*self.parents, *_upgrade_variables(parents)),
             )
         elif parents.is_conditioned():
-            raise TypeError('can not be given a distribution that has conditionals')
+            raise TypeError("can not be given a distribution that has conditionals")
         else:
             # The parents variable is actually a Distribution instance with no parents,
             #  so its children get appended as parents for the new mixed distribution
             return Distribution(
                 children=self.children,
-                parents=(*self.parents, *parents.children),  # don't think about this too hard
+                parents=(
+                    *self.parents,
+                    *parents.children,
+                ),  # don't think about this too hard
             )
 
     def __or__(self, parents: XSeq[Variable]) -> Distribution:
@@ -380,14 +428,14 @@ class Probability(Expression):
 
     def to_text(self) -> str:
         """Output this probability in the internal string format."""
-        return f'P({self.distribution.to_text()})'
+        return f"P({self.distribution.to_text()})"
 
     def to_latex(self) -> str:
         """Output this probability in the LaTeX string format."""
-        return f'P({self.distribution.to_latex()})'
+        return f"P({self.distribution.to_latex()})"
 
     def __repr__(self):
-        return f'P({repr(self.distribution)})'
+        return f"P({repr(self.distribution)})"
 
     def __mul__(self, other: Expression) -> Expression:
         if isinstance(other, Product):
@@ -400,6 +448,27 @@ class Probability(Expression):
     def __truediv__(self, expression: Expression) -> Fraction:
         return Fraction(self, expression)
 
+    def uncondition(self) -> Probability:
+        """Return a new probability where the underlying distribution is no longer conditioned by the parents.
+
+        :returns: A new probability over a distribution over the children and parents of the previous distribution
+
+        >>> from y0.dsl import P, A, B
+        >>> P(A | B).uncondition() == P(A, B)
+        """
+        return Probability(self.distribution.uncondition())
+
+    def marginalize(self, ranges: XSeq[Variable]) -> Fraction:
+        """Return this probability distribution, marginalized by the given variables.
+
+        :param ranges: A variable or list of variables over which to marganilize this probability's distribution
+        :returns: A fraction in which the denominator is represents the sum over the given ranges
+
+        >>> from y0.dsl import P, A, B
+        >>> P(A, B).marginalize(A) == P(A, B) / Sum[A](P(A, B))
+        """
+        return self / Sum(expression=self, ranges=_prepare_ranges(ranges))
+
     def _iter_variables(self) -> Iterable[Variable]:
         """Get the set of variables used in the distribution in this probability."""
         yield from self.distribution._iter_variables()
@@ -470,11 +539,11 @@ class Product(Expression):
 
     def to_text(self):
         """Output this product in the internal string format."""
-        return ' '.join(expression.to_text() for expression in self.expressions)
+        return " ".join(expression.to_text() for expression in self.expressions)
 
     def to_latex(self):
         """Output this product in the LaTeX string format."""
-        return ' '.join(expression.to_latex() for expression in self.expressions)
+        return " ".join(expression.to_latex() for expression in self.expressions)
 
     def __mul__(self, other: Expression):
         if isinstance(other, Product):
@@ -494,11 +563,11 @@ class Product(Expression):
 
 
 def _list_to_text(elements: Iterable[_Mathable]) -> str:
-    return ','.join(element.to_text() for element in elements)
+    return ",".join(element.to_text() for element in elements)
 
 
 def _list_to_latex(elements: Iterable[_Mathable]) -> str:
-    return ','.join(element.to_latex() for element in elements)
+    return ",".join(element.to_latex() for element in elements)
 
 
 @dataclass(frozen=True)
@@ -513,12 +582,12 @@ class Sum(Expression):
     def to_text(self) -> str:
         """Output this sum in the internal string format."""
         ranges = _list_to_text(self.ranges)
-        return f'[ sum_{{{ranges}}} {self.expression.to_text()} ]'
+        return f"[ sum_{{{ranges}}} {self.expression.to_text()} ]"
 
     def to_latex(self) -> str:
         """Output this sum in the LaTeX string format."""
         ranges = _list_to_latex(self.ranges)
-        return rf'\sum_{{{ranges}}} {self.expression.to_latex()}'
+        return rf"\sum_{{{ranges}}} {self.expression.to_latex()}"
 
     def __mul__(self, expression: Expression):
         if isinstance(expression, Product):
@@ -536,7 +605,9 @@ class Sum(Expression):
             yield from variable._iter_variables()
 
     @classmethod
-    def __class_getitem__(cls, ranges: Union[Variable, Tuple[Variable, ...]]) -> Callable[[Expression], Sum]:
+    def __class_getitem__(
+        cls, ranges: Union[Variable, Tuple[Variable, ...]]
+    ) -> Callable[[Expression], Sum]:
         """Create a partial sum object over the given ranges.
 
         :param ranges: The variables over which the partial sum will be done
@@ -555,13 +626,12 @@ class Sum(Expression):
         return functools.partial(Sum, ranges=_prepare_ranges(ranges))
 
 
-def _prepare_ranges(ranges: Union[Variable, Tuple[Variable, ...]]) -> Tuple[Variable, ...]:
-    if isinstance(ranges, tuple):
-        return tuple(ranges)
-    elif isinstance(ranges, Variable):  # a single element is not given as a tuple, such as in Sum[T]
+def _prepare_ranges(ranges: XSeq[Variable]) -> Tuple[Variable, ...]:
+    if isinstance(
+        ranges, Variable
+    ):  # a single element is not given as a tuple, such as in Sum[T]
         return (ranges,)
-    else:
-        raise TypeError
+    return tuple(ranges)
 
 
 @dataclass(frozen=True)
@@ -575,21 +645,29 @@ class Fraction(Expression):
 
     def to_text(self) -> str:
         """Output this fraction in the internal string format."""
-        return f'frac_{{{self.numerator.to_text()}}}{{{self.denominator.to_text()}}}'
+        return f"frac_{{{self.numerator.to_text()}}}{{{self.denominator.to_text()}}}"
 
     def to_latex(self) -> str:
         """Output this fraction in the LaTeX string format."""
-        return rf'\frac{{{self.numerator.to_latex()}}}{{{self.denominator.to_latex()}}}'
+        return rf"\frac{{{self.numerator.to_latex()}}}{{{self.denominator.to_latex()}}}"
 
     def __mul__(self, expression: Expression) -> Fraction:
         if isinstance(expression, Fraction):
-            return Fraction(self.numerator * expression.numerator, self.denominator * expression.denominator)
+            return Fraction(
+                self.numerator * expression.numerator,
+                self.denominator * expression.denominator,
+            )
         else:
             return Fraction(self.numerator * expression, self.denominator)
 
     def __truediv__(self, expression: Expression) -> Fraction:
         if isinstance(expression, Fraction):
-            return Fraction(self.numerator * expression.denominator, self.denominator * expression.numerator)
+            return Fraction(
+                self.numerator * expression.denominator,
+                self.denominator * expression.numerator,
+            )
+        elif isinstance(expression, One):
+            return self
         else:
             return Fraction(self.numerator, self.denominator * expression)
 
@@ -598,17 +676,96 @@ class Fraction(Expression):
         yield from self.numerator._iter_variables()
         yield from self.denominator._iter_variables()
 
+    def simplify(self) -> Expression:
+        """Simplify this fraction."""
+        if isinstance(self.denominator, One):
+            return self.numerator
+        if isinstance(self.numerator, One):
+            return self
+        if self.numerator == self.denominator:
+            return One()
+        if isinstance(self.numerator, Product) and isinstance(
+            self.denominator, Product
+        ):
+            return self._simplify_parts(
+                self.numerator.expressions, self.denominator.expressions
+            )
+        elif isinstance(self.numerator, Product):
+            return self._simplify_parts(self.numerator.expressions, [self.denominator])
+        elif isinstance(self.denominator, Product):
+            return self._simplify_parts([self.numerator], self.denominator.expressions)
+        return self
+
+    @classmethod
+    def _simplify_parts(
+        cls, numerator: Sequence[Expression], denominator: Sequence[Expression]
+    ) -> Expression:
+        """Calculate the minimum fraction.
+
+        :param numerator: A sequence of expressions that are multiplied in the product in the numerator
+        :param denominator: A sequence of expressions that are multiplied in the product in the denominator
+        :returns: A simplified fraction.
+        """
+        new_numerator, new_denominator = cls._simplify_parts_helper(
+            numerator, denominator
+        )
+        if new_numerator and new_denominator:
+            return Fraction(
+                _expression_or_product(new_numerator),
+                _expression_or_product(new_denominator),
+            )
+        elif new_numerator:
+            return _expression_or_product(new_numerator)
+        elif new_denominator:
+            return One() / _expression_or_product(new_denominator)
+        else:
+            return One()
+
+    @staticmethod
+    def _simplify_parts_helper(
+        numerator: Sequence[Expression],
+        denominator: Sequence[Expression],
+    ) -> Tuple[Tuple[Expression, ...], Tuple[Expression, ...]]:
+        numerator_cancelled = set()
+        denominator_cancelled = set()
+        for i, n_expr in enumerate(numerator):
+            for j, d_expr in enumerate(denominator):
+                if j in denominator_cancelled:
+                    continue
+                if n_expr == d_expr:
+                    numerator_cancelled.add(i)
+                    denominator_cancelled.add(j)
+                    break
+        return (
+            tuple(
+                expr for i, expr in enumerate(numerator) if i not in numerator_cancelled
+            ),
+            tuple(
+                expr
+                for i, expr in enumerate(denominator)
+                if i not in denominator_cancelled
+            ),
+        )
+
+
+def _expression_or_product(e: Sequence[Expression]) -> Expression:
+    if not e:
+        raise ValueError
+    if 1 == len(e):
+        return e[0]
+    return Product(tuple(e))
+
 
 class One(Expression):
     """The multiplicative identity (1)."""
 
     def to_text(self) -> str:
         """Output this identity variable in the internal string format."""
-        return '1'
+        return "1"
 
     def to_latex(self) -> str:
         """Output this identity instance in the LaTeX string format."""
-        return '1'
+        return "1"
 
     def __rmul__(self, expression: Expression) -> Expression:
         return expression
@@ -619,6 +776,9 @@ class One(Expression):
     def __truediv__(self, other: Expression) -> Fraction:
         return Fraction(self, other)
 
+    def __eq__(self, other):
+        return isinstance(other, One)  # all ones are equal
+
     def _iter_variables(self) -> Iterable[Variable]:
         """Get the set of variables used in this expression."""
         return iter([])
@@ -658,13 +818,13 @@ class QFactor(Expression):
         """Output this fraction in the internal string format."""
         codomain = _list_to_latex(self.codomain)
         domain = _list_to_text(self.domain)
-        return f'Q[{codomain}]({domain})'
+        return f"Q[{codomain}]({domain})"
 
     def to_latex(self) -> str:
         """Output this fraction in the LaTeX string format."""
         codomain = _list_to_latex(self.codomain)
         domain = _list_to_text(self.domain)
-        return rf'Q_{{{codomain}}}({{{domain}}})'
+        return rf"Q_{{{codomain}}}({{{domain}}})"
 
     def __mul__(self, other: Expression):
         if isinstance(other, Product):
@@ -687,13 +847,33 @@ class QFactor(Expression):
 
 Q = QFactor
 
-A, B, C, D, R, S, T, W, X, Y, Z = map(Variable, 'ABCDRSTWXYZ')  # type: ignore
-V1, V2, V3, V4, V5, V6 = [Variable(f'V{i}') for i in range(1, 7)]
-Z1, Z2, Z3, Z4, Z5, Z6 = [Variable(f'Z{i}') for i in range(1, 7)]
+A, B, C, D, R, S, T, W, X, Y, Z = map(Variable, "ABCDRSTWXYZ")  # type: ignore
+V1, V2, V3, V4, V5, V6 = [Variable(f"V{i}") for i in range(1, 7)]
+Z1, Z2, Z3, Z4, Z5, Z6 = [Variable(f"Z{i}") for i in range(1, 7)]
 
 
 def _upgrade_ordering(variables: Iterable[Union[str, Variable]]) -> Sequence[Variable]:
-    return tuple(
-        Variable(variable) if isinstance(variable, str) else variable
-        for variable in variables
-    )
+    return tuple(Variable.norm(variable) for variable in variables)
+
+
+OrderingHint = Optional[Sequence[Union[str, Variable]]]
+
+
+def ensure_ordering(
+    expression: Expression,
+    *,
+    ordering: OrderingHint = None,
+) -> Sequence[Variable]:
+    """Get a canonical ordering of the variables in the expression, or pass one through.
+
+    The canonical ordering of the variables in a given expression is based on the alphabetical
+    sort order of the variables based on their names.
+
+    :param expression: The expression to get a canonical ordering from.
+    :param ordering: A given ordering to pass through if not none, otherwise calculate it.
+    :returns: The ordering
+    """
+    if ordering is not None:
+        return _upgrade_ordering(ordering)
+    # use alphabetical ordering
+    return sorted(expression.get_variables(), key=attrgetter("name"))
