@@ -5,7 +5,7 @@
 from .dsl import Expression, Fraction, Probability, Product, Sum
 
 __all__ = [
-    'has_markov_postcondition',
+    "has_markov_postcondition",
 ]
 
 
@@ -20,12 +20,13 @@ def has_markov_postcondition(expression: Expression) -> bool:
         return expression.distribution.is_markov_kernel()
     elif isinstance(expression, Product):
         return all(
-            has_markov_postcondition(subexpr)
-            for subexpr in expression.expressions
+            has_markov_postcondition(subexpr) for subexpr in expression.expressions
         )
     elif isinstance(expression, Sum):
         return has_markov_postcondition(expression.expression)
     elif isinstance(expression, Fraction):
-        return has_markov_postcondition(expression.numerator) and has_markov_postcondition(expression.denominator)
+        return has_markov_postcondition(
+            expression.numerator
+        ) and has_markov_postcondition(expression.denominator)
     else:
         raise TypeError
